@@ -3,9 +3,9 @@ MakeCatalog - Make a catalog from an input and labeled image.
 MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -47,18 +47,17 @@ enum program_args_groups
 
 /* Available letters for short options:
 
-   f g k l u v w x y z
-   H J L W X Y
+   f g k w x y z
+   E H J L O R W X Y
 */
 enum option_keys_enum
 {
   /* With short-option version. */
-  UI_KEY_OBJECTSFILE     = 'O',         /* General settings. */
-  UI_KEY_CLUMPSFILE      = 'C',
-  UI_KEY_SKYFILE         = 's',
-  UI_KEY_STDFILE         = 't',
-  UI_KEY_SKYSUBTRACTED   = 'E',
-  UI_KEY_THRESHOLD       = 'R',
+  UI_KEY_CLUMPSCAT       = 'C',         /* General settings. */
+  UI_KEY_VALUESFILE      = 'v',
+  UI_KEY_CLUMPSFILE      = 'l',
+  UI_KEY_INSKY           = 's',
+  UI_KEY_INSTD           = 't',
   UI_KEY_ENVSEED         = 'e',
 
   UI_KEY_IDS             = 'i',         /* Catalog columns. */
@@ -81,11 +80,14 @@ enum option_keys_enum
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
-  UI_KEY_OBJECTSHDU      = 1000,        /* General settings. */
+  UI_KEY_VALUESHDU       = 1000,        /* General settings. */
   UI_KEY_CLUMPSHDU,
   UI_KEY_SKYHDU,
   UI_KEY_STDHDU,
+  UI_KEY_WITHCLUMPS,
   UI_KEY_ZEROPOINT,
+  UI_KEY_VARIANCE,
+  UI_KEY_SUBTRACTSKY,
   UI_KEY_SFMAGNSIGMA,
   UI_KEY_SFMAGAREA,
   UI_KEY_UPMASKFILE,
@@ -94,17 +96,24 @@ enum option_keys_enum
   UI_KEY_UPRANGE,
   UI_KEY_UPSIGMACLIP,
   UI_KEY_UPNSIGMA,
+  UI_KEY_CHECKUPLIM,
+  UI_KEY_NOCLUMPSORT,
 
   UI_KEY_OBJID,                         /* Catalog columns. */
   UI_KEY_IDINHOSTOBJ,
   UI_KEY_CLUMPSAREA,
   UI_KEY_WEIGHTAREA,
+  UI_KEY_GEOAREA,
   UI_KEY_GEOX,
   UI_KEY_GEOY,
   UI_KEY_CLUMPSX,
   UI_KEY_CLUMPSY,
   UI_KEY_CLUMPSGEOX,
   UI_KEY_CLUMPSGEOY,
+  UI_KEY_MINX,
+  UI_KEY_MAXX,
+  UI_KEY_MINY,
+  UI_KEY_MAXY,
   UI_KEY_W1,
   UI_KEY_W2,
   UI_KEY_GEOW1,
@@ -113,10 +122,17 @@ enum option_keys_enum
   UI_KEY_CLUMPSW2,
   UI_KEY_CLUMPSGEOW1,
   UI_KEY_CLUMPSGEOW2,
+  UI_KEY_BRIGHTNESSERR,
   UI_KEY_CLUMPSBRIGHTNESS,
-  UI_KEY_NORIVERBRIGHTNESS,
+  UI_KEY_BRIGHTNESSNORIVER,
+  UI_KEY_MEAN,
+  UI_KEY_MEDIAN,
   UI_KEY_CLUMPSMAGNITUDE,
   UI_KEY_UPPERLIMIT,
+  UI_KEY_UPPERLIMITONESIGMA,
+  UI_KEY_UPPERLIMITSIGMA,
+  UI_KEY_UPPERLIMITQUANTILE,
+  UI_KEY_UPPERLIMITSKEW,
   UI_KEY_RIVERAVE,
   UI_KEY_RIVERNUM,
   UI_KEY_SKY,
