@@ -1,11 +1,11 @@
 /*********************************************************************
-NoiseChisel - Detect and segment signal in noise.
+NoiseChisel - Detect signal in a noisy dataset.
 NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -24,9 +24,9 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #define DETECTION_H
 
 void
-initialdetection(struct noisechiselparams *p);
+detection_initial(struct noisechiselparams *p);
 
 void
-onlytruedetections(struct noisechiselparams *p);
+detection(struct noisechiselparams *p);
 
 #endif
