@@ -1,11 +1,11 @@
 /*********************************************************************
-NoiseChisel - Detect and segment signal in a noisy dataset.
+NoiseChisel - Detect signal in a noisy dataset.
 NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -30,8 +30,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gnuastro/wcs.h>
 #include <gnuastro/fits.h>
+#include <gnuastro/array.h>
 #include <gnuastro/blank.h>
 #include <gnuastro/threads.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/dimension.h>
 
 #include <gnuastro-internal/timing.h>
@@ -66,8 +68,8 @@ args_doc[] = "ASTRdata";
 const char
 doc[] = GAL_STRINGS_TOP_HELP_INFO PROGRAM_NAME" Detects and segments signal "
   "that is deeply burried in noise. It employs a noise-based detection and "
-  "segmentation method enabling it to be very resilient to the rich diversity "
-  "of shapes in astronomical targets.\n"
+  "segmentation method enabling it to be very resilient to the rich "
+  "diversity of shapes in astronomical targets.\n"
   GAL_STRINGS_MORE_HELP_INFO
   /* After the list of options: */
   "\v"
@@ -217,8 +219,8 @@ ui_read_check_only_options(struct noisechiselparams *p)
 {
   /* If the convolved option is given, then the convolved HDU is also
      mandatory. */
-  if(p->convolvedname && p->convolvedhdu==NULL)
-    error(EXIT_FAILURE, 0, "no value given to `--convolvedhdu'. When the "
+  if(p->convolvedname && p->chdu==NULL)
+    error(EXIT_FAILURE, 0, "no value given to `--chdu'. When the "
           "`--convolved' option is called (to specify a convolved image "
           "and avoid convolution) it is mandatory to also specify a HDU "
           "for it");
@@ -241,16 +243,16 @@ ui_read_check_only_options(struct noisechiselparams *p)
 
   /* For the options that make tables, the table formation option is
      mandatory. */
-  if( (p->checkdetsn || p->checkclumpsn) && p->cp.tableformat==0 )
+  if( p->checksn && p->cp.tableformat==0 )
     error(EXIT_FAILURE, 0, "`--tableformat' is necessary with the "
-          "`--checkdetsn' and `--checkclumpsn' options.\n"
+          "`--checksn' option.\n"
           "Please see description for `--tableformat' after running the "
           "following command for more information (use `SPACE' to go down "
           "the page and `q' to return to the command-line):\n\n"
           "    $ info gnuastro \"Input Output options\"");
 
   /* Kernel checks. */
-  if(p->kernelname)
+  if(p->kernelname && strcmp(p->kernelname, UI_NO_CONV_KERNEL_NAME))
     {
       /* Check if it exists. */
       gal_checkset_check_file(p->kernelname);
@@ -271,12 +273,12 @@ ui_read_check_only_options(struct noisechiselparams *p)
       gal_checkset_check_file(p->widekernelname);
 
       /* If its FITS, see if a HDU has been provided. */
-      if( gal_fits_name_is_fits(p->widekernelname) && p->wkhdu==NULL )
-        error(EXIT_FAILURE, 0, "no HDU specified for wide kernel. When the "
-              "wide kernel is a FITS file, a HDU must also be specified. You "
-              "can use the `--khdu' option and give it the HDU number "
-              "(starting from zero), extension name, or anything "
-              "acceptable by CFITSIO");
+      if( gal_fits_name_is_fits(p->widekernelname) && p->whdu==NULL )
+        error(EXIT_FAILURE, 0, "no HDU specified for the given wide kernel "
+              "(`%s'). When the wide kernel is a FITS file, a HDU must also "
+              "be specified. You can use the `--whdu' option and give it the "
+              "HDU number (starting from zero), extension name, or any "
+              "HDU identifier acceptable by CFITSIO", p->widekernelname);
     }
 }
 
@@ -346,7 +348,7 @@ ui_set_output_names(struct noisechiselparams *p)
     }
   else
     p->cp.output=gal_checkset_automatic_output(&p->cp, p->inputname,
-                                               "_labeled.fits");
+                                               "_detected.fits");
 
   /* Tile check. */
   if(p->cp.tl.checktiles)
@@ -364,7 +366,7 @@ ui_set_output_names(struct noisechiselparams *p)
                                                 "_detsky.fits");
 
   /* Pseudo-detection S/N values. */
-  if(p->checkdetsn)
+  if(p->checksn)
     {
       p->detsn_s_name=gal_checkset_automatic_output(&p->cp, basename,
                  ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
@@ -380,27 +382,11 @@ ui_set_output_names(struct noisechiselparams *p)
   /* Detection steps. */
   if(p->checkdetection)
     p->detectionname=gal_checkset_automatic_output(&p->cp, basename,
-                                               "_det.fits");
+                                               "_detcheck.fits");
 
-  /* Detection steps. */
+  /* Sky checks. */
   if(p->checksky)
     p->skyname=gal_checkset_automatic_output(&p->cp, basename, "_sky.fits");
-
-  /* Clump S/N values. */
-  if(p->checkclumpsn)
-    {
-      p->clumpsn_s_name=gal_checkset_automatic_output(&p->cp, basename,
-                 ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
-                   ? "_clumpsn_sky.txt" : "_clumpsn_sky.fits") );
-      p->clumpsn_d_name=gal_checkset_automatic_output(&p->cp, basename,
-                 ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
-                   ? "_clumpsn_det.txt" : "_clumpsn_det.fits") );
-    }
-
-  /* Segmentation steps. */
-  if(p->checksegmentation)
-    p->segmentationname=gal_checkset_automatic_output(&p->cp, basename,
-                                                      "_seg.fits");
 }
 
 
@@ -410,64 +396,21 @@ ui_set_output_names(struct noisechiselparams *p)
 static void
 ui_prepare_kernel(struct noisechiselparams *p)
 {
-  /* The default kernel. It was created by saving the following commands in a
-     script and running it. It will create a plain text array along with a
-     FITS image. The crop is because the first and last rows and columns of
-     all PSFs made by MakeProfiles are blank (zero) when you run with
-     oversample=1. You can keep the spaces when copying and pasting ;-). Just
-     make it executable and run it.
+  float *f, *ff, *k;
 
-     set -o errexit           # Stop if a program returns false.
-     echo "0 0.0 0.0 3 2 0 0 1 1 5" > tmp.txt
-     export GSL_RNG_TYPE=ranlxs2
-     export GSL_RNG_SEED=1
-     astmkprof tmp.txt --oversample=1 --envseed --numrandom=10000 \
-     --tolerance=0.01 --nomerged
-     astcrop 0_tmp.fits --section=2:*-1,2:*-1 --zeroisnotblank    \
-     --output=fwhm2.fits
-     astconvertt fwhm2.fits --output=fwhm2.txt
-     rm 0_tmp.fits tmp.txt
-  */
-  size_t kernel_2d_dsize[2]={11,11};
-  float *f, *ff, *k, kernel_2d[121]=
-    {
-      0, 0, 0, 0, 0, 6.57699e-09, 0, 0, 0, 0, 0,
-
-      0, 0, 6.57699e-09, 2.10464e-07, 1.68371e-06, 3.36742e-06, 1.68371e-06,
-      2.10464e-07, 6.57699e-09, 0, 0,
-
-      0, 6.57699e-09, 8.41855e-07, 2.69394e-05, 0.000383569, 0.000717224,
-      0.000379782, 2.69394e-05, 8.41855e-07, 6.57699e-09, 0,
-
-      0, 2.10464e-07, 2.69394e-05, 0.00140714, 0.00888549, 0.016448,
-      0.00867408, 0.00138203, 2.69394e-05, 2.10464e-07, 0,
-
-      0, 1.68371e-06, 0.000381138, 0.00875434, 0.0573377, 0.106308, 0.0570693,
-      0.00891745, 0.000378914, 1.68371e-06, 0,
-
-      6.57699e-09, 3.36742e-06, 0.00071364, 0.0164971, 0.106865, 0.197316,
-      0.106787, 0.0166434, 0.000713827, 3.36742e-06, 6.57699e-09,
-
-      0, 1.68371e-06, 0.000215515, 0.00894112, 0.0573699, 0.106239, 0.0567907,
-      0.00901191, 0.000215515, 1.68371e-06, 0,
-
-      0, 2.10464e-07, 2.69394e-05, 0.00135085, 0.0089288, 0.0164171,
-      0.00879334, 0.0013622, 2.69394e-05, 2.10464e-07, 0,
-
-      0, 6.57699e-09, 8.41855e-07, 2.69394e-05, 0.000215515, 0.000724137,
-      0.000215515, 2.69394e-05, 8.41855e-07, 6.57699e-09, 0,
-
-      0, 0, 6.57699e-09, 2.10464e-07, 1.68371e-06, 3.36742e-06, 1.68371e-06,
-      2.10464e-07, 6.57699e-09, 0, 0,
-
-      0, 0, 0, 0, 0, 6.57699e-09, 0, 0, 0, 0, 0
-    };
+/* Impor the default kernel. */
+#include "kernel-2d.h"
 
   /* If a kernel file is given, then use it. Otherwise, use the default
      kernel. */
   if(p->kernelname)
-    p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
-                                       p->cp.minmapsize);
+    {
+      if( strcmp(p->kernelname, UI_NO_CONV_KERNEL_NAME) )
+        p->kernel=gal_fits_img_read_kernel(p->kernelname, p->khdu,
+                                           p->cp.minmapsize);
+      else
+        p->kernel=NULL;
+    }
   else
     {
       /* Allocate space for the kernel (we don't want to use the statically
@@ -486,7 +429,7 @@ ui_prepare_kernel(struct noisechiselparams *p)
   /* If a wide kernel is given, then read it into memory. Otherwise, just
      ignore it. */
   if(p->widekernelname)
-    p->widekernel=gal_fits_img_read_kernel(p->widekernelname, p->wkhdu,
+    p->widekernel=gal_fits_img_read_kernel(p->widekernelname, p->whdu,
                                            p->cp.minmapsize);
 }
 
@@ -504,8 +447,8 @@ ui_prepare_tiles(struct noisechiselparams *p)
   /* Check the tile parameters for the small tile sizes and make the tile
      structure. We will also need the dimensions of the tile with the
      maximum required memory. */
-  p->maxtsize=gal_data_malloc_array(GAL_TYPE_SIZE_T, p->input->ndim,
-                                    __func__, "p->maxtsize");
+  p->maxtsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, p->input->ndim, 0,
+                                   __func__, "p->maxtsize");
   gal_tile_full_sanity_check(p->inputname, p->cp.hdu, p->input, tl);
   gal_tile_full_two_layers(p->input, tl);
   gal_tile_full_permutation(tl);
@@ -524,8 +467,8 @@ ui_prepare_tiles(struct noisechiselparams *p)
   ltl->workoverch     = tl->workoverch;
   ltl->checktiles     = tl->checktiles;
   ltl->oneelempertile = tl->oneelempertile;
-  p->maxltsize=gal_data_malloc_array(GAL_TYPE_SIZE_T, p->input->ndim,
-                                     __func__, "p->maxltsize");
+  p->maxltsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, p->input->ndim, 0,
+                                    __func__, "p->maxltsize");
   gal_tile_full_sanity_check(p->inputname, p->cp.hdu, p->input, ltl);
   gal_tile_full_two_layers(p->input, ltl);
   gal_tile_full_permutation(ltl);
@@ -573,17 +516,18 @@ ui_prepare_tiles(struct noisechiselparams *p)
 
 
 
+/* Read the input image and do the basic checks */
 static void
-ui_preparations(struct noisechiselparams *p)
+ui_preparations_read_input(struct noisechiselparams *p)
 {
-  /* Prepare the names of the outputs. */
-  ui_set_output_names(p);
-
+  float *f;
 
   /* Read the input as a single precision floating point dataset. */
-  p->input = gal_fits_img_read_to_type(p->inputname, p->cp.hdu,
-                                       GAL_TYPE_FLOAT32,
-                                       p->cp.minmapsize, 0, 0);
+  p->input = gal_array_read_one_ch_to_type(p->inputname, p->cp.hdu,
+                                           GAL_TYPE_FLOAT32,
+                                           p->cp.minmapsize);
+  p->input->wcs = gal_wcs_read(p->inputname, p->cp.hdu, 0, 0,
+                               &p->input->nwcs);
   if(p->input->name==NULL)
     gal_checkset_allocate_copy("INPUT", &p->input->name);
 
@@ -594,35 +538,69 @@ ui_preparations(struct noisechiselparams *p)
           "can only operate on 2D datasets (images)", p->inputname, p->cp.hdu,
           p->input->ndim);
 
+  /* A small check to see if the edges of the dataset aren't zero valued:
+     they should be masked. */
+  f=p->input->array;
+  if( (f[0]==0.0 && f[1]==0.0)
+      || (f[ p->input->size-1 ]==0.0 && f[ p->input->size-2 ]==0.0) )
+    error(0, 0, "%s (hdu %s): [*** WARNING ***]: The first and/or last few "
+          "pixels have a value of 0.0. As described below, the result of "
+          "this run may thus not be reasonable/optimal.\n\n"
+          "Some data reduction pipelines put 0.0 where there isn't data "
+          "(most commonly on the edges). However, NoiseChisel's "
+          "noise-based detection paradigm starts from the lower values of "
+          "the dataset (not high S/N peaks): its initial threshold is "
+          "mostly below the Sky value (0.0 in processed images). Therefore "
+          "0.0 is meaningful for NoiseChisel and must not be used for a "
+          "blank value.\n\n"
+          "To ignore certain pixels, they must have a blank/NaN value. "
+          "To mask (set to blank/NaN) the 0.0 valued elements, you can use "
+          "Gnuastro's Arithmetic program with a command like this:\n\n"
+          "    $ astarithmetic %s %s 0.0 eq nan where -g%s\n\n"
+          "If the few 0.0 valued pixels on the edges are meaningful for "
+          "your analysis, please ignore this warming message.\n"
+          "--------------------------",
+          p->inputname, p->cp.hdu, p->inputname, p->inputname, p->cp.hdu);
+}
+
+
+
+
+
+static void
+ui_preparations(struct noisechiselparams *p)
+{
+  /* Prepare the names of the outputs. */
+  ui_set_output_names(p);
+
+  /* Read the input datasets and do the basic checks.*/
+  ui_preparations_read_input(p);
 
   /* If a convolved image was given, read it in. Otherwise, read the given
      kernel. */
   if(p->convolvedname)
     {
       /* Read the input convolved image. */
-      p->conv = gal_fits_img_read_to_type(p->convolvedname, p->convolvedhdu,
-                                          GAL_TYPE_FLOAT32, p->cp.minmapsize,
-                                          0, 0);
+      p->conv = gal_array_read_one_ch_to_type(p->convolvedname, p->chdu,
+                                              GAL_TYPE_FLOAT32,
+                                              p->cp.minmapsize);
 
       /* Make sure the convolved image is the same size as the input. */
-      if( gal_data_dsize_is_different(p->input, p->conv) )
+      if( gal_dimension_is_different(p->input, p->conv) )
         error(EXIT_FAILURE, 0, "%s (hdu %s), given to `--convolved' and "
               "`--convolvehdu', is not the same size as NoiseChisel's "
-              "input: %s (hdu: %s)", p->convolvedname, p->convolvedhdu,
+              "input: %s (hdu: %s)", p->convolvedname, p->chdu,
               p->inputname, p->cp.hdu);
     }
   else
     ui_prepare_kernel(p);
 
-
   /* Check for blank values to help later processing.  */
   gal_blank_present(p->input, 1);
 
-
   /* Prepare the tessellation. */
   ui_prepare_tiles(p);
 
-
   /* Allocate space for the over-all necessary arrays. */
   p->binary=gal_data_alloc(NULL, GAL_TYPE_UINT8, p->input->ndim,
                            p->input->dsize, p->input->wcs, 0,
@@ -716,20 +694,25 @@ ui_read_check_inputs_setup(int argc, char *argv[],
       printf("  - Input: %s (hdu: %s)\n", p->inputname, p->cp.hdu);
       if(p->convolvedname)
         printf("  - Convolved input: %s (hdu: %s)\n",
-               p->convolvedname, p->convolvedhdu);
+               p->convolvedname, p->chdu);
       else
         {
           if(p->kernelname)
-            printf("  - %s: %s (hdu: %s)\n",
-                   p->widekernelname ? "Sharp Kernel" : "Kernel",
-                   p->kernelname, p->khdu);
+            {
+              if( strcmp(p->kernelname, UI_NO_CONV_KERNEL_NAME) )
+                printf("  - %s: %s (hdu: %s)\n",
+                       p->widekernelname ? "Sharp Kernel" : "Kernel",
+                       p->kernelname, p->khdu);
+              else
+                printf("  - No convolution requested.\n");
+            }
           else
             printf("  - %s: FWHM=2 pixel Gaussian.\n",
                    p->widekernelname ? "Sharp Kernel" : "Kernel");
         }
       if(p->widekernelname)
         printf("  - Wide Kernel: %s (hdu: %s)\n", p->widekernelname,
-               p->wkhdu);
+               p->whdu);
     }
 }
 
@@ -761,8 +744,16 @@ ui_abort_after_check(struct noisechiselparams *p, char *filename,
 {
   char *name;
 
-  if(file2name) asprintf(&name, "`%s' and `%s'", filename, file2name);
-  else          asprintf(&name, "`%s'", filename);
+  if(file2name)
+    {
+      if( asprintf(&name, "`%s' and `%s'", filename, file2name)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+    }
+  else
+    {
+      if( asprintf(&name, "`%s'", filename)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+    }
 
   /* Let the user know that NoiseChisel is aborting. */
   fprintf(stderr,
@@ -802,21 +793,17 @@ ui_free_report(struct noisechiselparams *p, struct timeval *t1)
   if(p->detsn_s_name)     free(p->detsn_s_name);
   if(p->detsn_d_name)     free(p->detsn_d_name);
   if(p->detectionname)    free(p->detectionname);
-  if(p->clumpsn_s_name)   free(p->clumpsn_s_name);
-  if(p->clumpsn_d_name)   free(p->clumpsn_d_name);
-  if(p->segmentationname) free(p->segmentationname);
 
   /* Free the allocated datasets. */
   gal_data_free(p->sky);
   gal_data_free(p->std);
-  gal_data_free(p->conv);
   gal_data_free(p->wconv);
   gal_data_free(p->input);
   gal_data_free(p->kernel);
   gal_data_free(p->binary);
   gal_data_free(p->olabel);
-  gal_data_free(p->clabel);
   gal_data_free(p->widekernel);
+  if(p->conv!=p->input) gal_data_free(p->conv);
 
   /* Clean up the tile structure. */
   p->ltl.numchannels=NULL;
