@@ -3,9 +3,9 @@ Statistics - Statistical analysis on input dataset.
 Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,6 +33,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/fits.h>
 #include <gnuastro/tile.h>
 #include <gnuastro/blank.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/arithmetic.h>
 #include <gnuastro/statistics.h>
 #include <gnuastro/interpolate.h>
@@ -61,7 +62,7 @@ statistics_pull_out_element(gal_data_t *input, size_t index)
   gal_data_t *out=gal_data_alloc(NULL, input->type, 1, &dsize,
                                  NULL, 1, -1, NULL, NULL, NULL);
   memcpy( out->array,
-          gal_data_ptr_increment(input->array, index, input->type),
+          gal_pointer_increment(input->array, index, input->type),
           gal_type_sizeof(input->type) );
   return out;
 }
@@ -97,7 +98,6 @@ statistics_print_one_row(struct statisticsparams *p)
   gal_data_t *tmpv, *out=NULL, *num=NULL, *min=NULL, *max=NULL;
   gal_data_t *sum=NULL, *med=NULL, *meanstd=NULL, *modearr=NULL;
 
-
   /* The user can ask for any of the operators more than once, also some
      operators might return more than one usable value (like mode). So we
      will calculate the desired values once, and then print them. */
@@ -254,7 +254,7 @@ statistics_interpolate_and_write(struct statisticsparams *p,
     }
 
   /* Write the values. */
-  gal_tile_full_values_write(values, &cp->tl, output, NULL, PROGRAM_NAME);
+  gal_tile_full_values_write(values, &cp->tl, 1, output, NULL, PROGRAM_NAME);
 }
 
 
@@ -303,8 +303,8 @@ statistics_on_tile(struct statisticsparams *p)
           type=GAL_TYPE_FLOAT64; break;
 
         default:
-          error(EXIT_FAILURE, 0, "%s: a bug! %d is not a recognized operation "
-                "code", __func__, operation->v);
+          error(EXIT_FAILURE, 0, "%s: a bug! %d is not a recognized "
+                "operation code", __func__, operation->v);
         }
 
       /* Allocate the space necessary to keep the value for each tile. */
@@ -385,7 +385,7 @@ statistics_on_tile(struct statisticsparams *p)
 
           /* Put the output value into the `values' array and clean up. */
           tmp=gal_data_copy_to_new_type_free(tmp, type);
-          memcpy(gal_data_ptr_increment(values->array, tind++, values->type),
+          memcpy(gal_pointer_increment(values->array, tind++, values->type),
                  tmp->array, gal_type_sizeof(type));
           gal_data_free(tmp);
         }
@@ -470,13 +470,37 @@ print_ascii_plot(struct statisticsparams *p, gal_data_t *plot,
 
 
 
+
+/* Data structure that must be fed into `gal_statistics_regular_bins'.*/
+static gal_data_t *
+set_bin_range_params(struct statisticsparams *p)
+{
+  size_t rsize=2;
+  gal_data_t *range=NULL;
+
+  if(p->manualbinrange)
+    {
+      /* Allocate the range data structure. */
+      range=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &rsize, NULL, 0, -1,
+                           NULL, NULL, NULL);
+      ((float *)(range->array))[0]=p->greaterequal;
+      ((float *)(range->array))[1]=p->lessthan;
+    }
+  return range;
+}
+
+
+
+
+
 static void
 ascii_plots(struct statisticsparams *p)
 {
-  gal_data_t *bins, *hist, *cfp=NULL;
+  gal_data_t *bins, *hist, *cfp=NULL, *range=NULL;
 
   /* Make the bins and the respective plot. */
-  bins=gal_statistics_regular_bins(p->input, NULL, p->numasciibins, NAN);
+  range=set_bin_range_params(p);
+  bins=gal_statistics_regular_bins(p->input, range, p->numasciibins, NAN);
   hist=gal_statistics_histogram(p->input, bins, 0, 0);
   if(p->asciicfp)
     {
@@ -538,7 +562,8 @@ write_output_table(struct statisticsparams *p, gal_data_t *table,
           ? gal_fits_name_is_fits(p->cp.output) ? "fits" : "txt"
           : "txt" );
   if(use_auto_output)
-    asprintf(&suffix, "%s.%s", suf, fix);
+    if( asprintf(&suffix, "%s.%s", suf, fix)<0 )
+      error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
 
 
   /* Make the output name. */
@@ -553,7 +578,8 @@ write_output_table(struct statisticsparams *p, gal_data_t *table,
   tmp=gal_fits_name_save_as_string(p->inputname, p->cp.hdu);
   gal_list_str_add(&comments, tmp, 0);
 
-  asprintf(&tmp, "%s created from:", contents);
+  if( asprintf(&tmp, "%s created from:", contents)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_list_str_add(&comments, tmp, 0);
 
   if(strcmp(fix, "fits"))  /* The intro info will be in FITS files anyway.*/
@@ -561,8 +587,8 @@ write_output_table(struct statisticsparams *p, gal_data_t *table,
 
 
   /* Write the table. */
-  gal_table_write(table, comments, p->cp.tableformat, output,
-                  p->cp.dontdelete);
+  gal_checkset_writable_remove(output, 0, p->cp.dontdelete);
+  gal_table_write(table, comments, p->cp.tableformat, output, "TABLE", 0);
 
 
   /* Let the user know, if we aren't in quiet mode. */
@@ -584,13 +610,13 @@ static void
 save_hist_and_or_cfp(struct statisticsparams *p)
 {
   char *suf, *contents;
-  gal_data_t *bins, *hist, *cfp=NULL;
-
+  gal_data_t *bins, *hist, *cfp=NULL, *range=NULL;
 
   /* Set the bins and make the histogram, this is necessary for both the
      histogram and CFP (recall that the CFP is built from the
      histogram). */
-  bins=gal_statistics_regular_bins(p->input, NULL, p->numbins,
+  range=set_bin_range_params(p);
+  bins=gal_statistics_regular_bins(p->input, range, p->numbins,
                                    p->onebinstart);
   hist=gal_statistics_histogram(p->input, bins, p->normalize, p->maxbinone);
 
@@ -634,6 +660,10 @@ save_hist_and_or_cfp(struct statisticsparams *p)
 
   /* Set the output file name. */
   write_output_table(p, bins, suf, contents);
+
+
+  /* Clean up. */
+  gal_data_free(range);
 }
 
 
@@ -720,12 +750,21 @@ print_input_info(struct statisticsparams *p)
   /* Range. */
   str=NULL;
   if( !isnan(p->greaterequal) && !isnan(p->lessthan) )
-    asprintf(&str, "from (inclusive) %g, up to (exclusive) %g",
-             p->greaterequal, p->lessthan);
+    {
+      if( asprintf(&str, "from (inclusive) %g, up to (exclusive) %g",
+                   p->greaterequal, p->lessthan)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+    }
   else if( !isnan(p->greaterequal) )
-    asprintf(&str, "from (inclusive) %g", p->greaterequal);
+    {
+      if( asprintf(&str, "from (inclusive) %g", p->greaterequal)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+    }
   else if( !isnan(p->lessthan) )
-    asprintf(&str, "up to (exclusive) %g", p->lessthan);
+    {
+      if( asprintf(&str, "up to (exclusive) %g", p->lessthan)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+    }
   if(str)
     {
       printf("Range: ");
@@ -759,7 +798,7 @@ print_basics(struct statisticsparams *p)
   int namewidth=40;
   float mirrdist=1.5;
   double mean, std, *d;
-  gal_data_t *tmp, *bins, *hist;
+  gal_data_t *tmp, *bins, *hist, *range=NULL;
 
   /* Define the input dataset. */
   print_input_info(p);
@@ -819,14 +858,16 @@ print_basics(struct statisticsparams *p)
      range of the histogram. In that case, we want to print the histogram
      information. */
   printf("-------");
+  range=set_bin_range_params(p);
   p->asciiheight = p->asciiheight ? p->asciiheight : 10;
   p->numasciibins = p->numasciibins ? p->numasciibins : 70;
-  bins=gal_statistics_regular_bins(p->input, NULL, p->numasciibins, NAN);
+  bins=gal_statistics_regular_bins(p->input, range, p->numasciibins, NAN);
   hist=gal_statistics_histogram(p->input, bins, 0, 0);
   if(p->refcol==NULL) printf("\nHistogram:\n");
   print_ascii_plot(p, hist, bins, 1, p->refcol ? 1 : 0);
   gal_data_free(bins);
   gal_data_free(hist);
+  gal_data_free(range);
 }
 
 
@@ -861,10 +902,16 @@ print_sigma_clip(struct statisticsparams *p)
 
   /* Set the mode for printing: */
   if( p->sclipparams[1]>=1.0f )
-    asprintf(&mode, "for %g clips", p->sclipparams[1]);
+    {
+      if( asprintf(&mode, "for %g clips", p->sclipparams[1])<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+    }
   else
-    asprintf(&mode, "until relative change in STD is less than %g",
-             p->sclipparams[1]);
+    {
+      if( asprintf(&mode, "until relative change in STD is less than %g",
+                   p->sclipparams[1])<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+    }
 
   /* Report the status */
   if(!p->cp.quiet)
@@ -969,5 +1016,4 @@ statistics(struct statisticsparams *p)
   /* If nothing was requested print the simple statistics. */
   if(print_basic_info)
     print_basics(p);
-
 }
