@@ -32,14 +32,15 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /* Available letters for short options:
 
-   a b d e f g j k l m n p r s t u v w x y z
+   a b d e f g j k l m n p r t u v w x y z
    A B C E G H J L O Q R W X Y
 */
 enum option_keys_enum
 {
   /* With short-option version. */
-  UI_KEY_COLUMN      = 'c',
-  UI_KEY_INFORMATION = 'i',
+  UI_KEY_COLUMN          = 'c',
+  UI_KEY_INFORMATION     = 'i',
+  UI_KEY_COLINFOINSTDOUT = 's',
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
