@@ -3,9 +3,9 @@ Box -- Define bounding and overlapping boxes.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -92,8 +92,8 @@ gal_box_bound_ellipse(double a, double b, double theta_deg, long *width)
      want the final height and width of the box enclosing the
      ellipse. So we have to multiply them by two, then take one from
      them (for the center). */
-  width[0] = 2 * ( (long)extent[0] + 1 ) + 1;
-  width[1] = 2 * ( (long)extent[1] + 1 ) + 1;
+  width[0] = 2 * ( (long)extent[0] ) + 1;
+  width[1] = 2 * ( (long)extent[1] ) + 1;
 }
 
 
