@@ -55,4 +55,8 @@ fi
 
 # Actual test script
 # ==================
-$execname $img --output=pdf --invert
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname $img --output=pdf --invert
