@@ -49,5 +49,10 @@ done
 
 # Actual test script
 # ==================
-$execname $img --center=0.99917157,1.0008283 --output=crop_wcscenter.fits \
-          --mode=wcs --width=0.3/3600
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname $img --center=0.99917157,1.0008283       \
+                              --output=crop_wcscenter.fits --mode=wcs  \
+                              --width=0.3/3600
